#pragma once
#include <stop_token>
#include <thread>

#include "util.hpp"

class EEPROM {
    public:
        EEPROM(const IOPins& ioPins, const AddressPins& addressPins, const Pin& pinCEb, const Pin& pinWEb, const Pin& pinOEb);
        inline void setRom(const std::array<uint8_t, AddressSize>& v) { rom = v; }
        inline void setRom(std::array<uint8_t, AddressSize>&& v) { rom = std::move(v); }
        inline const std::array<uint8_t, AddressSize>& getRom() const { return rom; }
        void simulate(std::stop_token token);
        ~EEPROM();
    private:
        std::array<uint8_t, AddressSize> rom;

        const IOPins& ioPins;
        const uint32_t ioMask;
        const AddressPins& addressPins;
        const Pin& pinCEb;
        const Pin& pinWEb;
        const Pin& pinOEb;

        void simulateThread(std::stop_token token);
        void setIODirection(uint8_t direction);
        uint8_t IODirection = INPUT;
        inline Address readAddress() { return readPins<Address>(this->addressPins); };
        inline uint8_t readData() { return readPins<uint8_t>(this->ioPins); };
};

