#pragma once

#include <cstdint>
#include <array>
#include <format>
#include <iostream>
#include <bcm2835.h>

const size_t AddressSize = 15;
using Pin = uint8_t;
using IOPins = std::array<Pin, 8>;
using AddressPins = std::array<Pin, AddressSize>;

using Address = uint16_t;

// TODO: remove when c++23 is used
namespace std {
    template <typename... Args>
    inline void println(const std::format_string<Args...> fmt, Args&&... args) {
        std::cout << std::vformat(fmt.get(), std::make_format_args(args...)) << std::endl;
    }
}

inline void waitForFallingEdge(uint8_t pin) {
    while (bcm2835_gpio_lev(pin) == LOW);
    while (bcm2835_gpio_lev(pin) == HIGH);
}
inline void waitForRisingEdge(uint8_t pin) {
    while (bcm2835_gpio_lev(pin) == HIGH);
    while (bcm2835_gpio_lev(pin) == LOW);
}
inline void waitForEdge(uint8_t pin) {
    if (bcm2835_gpio_lev(pin) == HIGH) {
        while (bcm2835_gpio_lev(pin) == HIGH);
        return;
    }
    else {
        while (bcm2835_gpio_lev(pin) == LOW);
        return;
    }
}

uint32_t byteToMask(char byte, const std::array<uint8_t, 8>& dataPins);

template<size_t N> 
constexpr uint32_t getMask(const std::array<Pin, N>& pins) {
    uint32_t mask = 0;
    for (size_t i = 0; i < pins.size(); i++) {
        /* const uint8_t bit = pins.at(i) - 1; */
        mask |= (1 << pins.at(i));
    }
    return mask;
}

inline uint8_t readPin(Pin pin) {
    return bcm2835_gpio_lev(pin);
}
template<typename T, size_t N> 
T readPins(const std::array<Pin, N>& pins) {
    static_assert(sizeof(T) * 8 >= N);
    T t = 0;
    for (size_t i = 0; i < pins.size(); i++) {
        t |= (bcm2835_gpio_lev(pins.at(i)) << i);
    }
    return t;
}

const uint8_t OUTPUT = BCM2835_GPIO_FSEL_OUTP;
const uint8_t INPUT = BCM2835_GPIO_FSEL_INPT;
